# -*- coding: utf-8 -*-
"""
Python 3.6
"""

from tkinter import *
from tkinter import ttk

from functools import partial
import socket


class SocketInterface():
    def __init__(
            self, HOST = '127.0.0.1', PORT = 65432, timeout = 3, 
            connection_refused_message = 'SOCKET_ERROR_REFUSED', timeout_message = 'SOCKET_ERROR_TIMEOUT', connection_reset_message = 'SOCKET_ERROR_RESET', 
            echo = True, prefix = b'', suffix = b''):

        self.HOST = HOST
        self.PORT = PORT
        self.timeout = timeout
        self.connection_refused_message = connection_refused_message
        self.timeout_message = timeout_message
        self.connection_reset_message = connection_reset_message
        self.echo = echo
        self.prefix = prefix
        self.suffix = suffix
        
    def server_send(self, message):
        message = b''.join((self.prefix, message.encode('utf-8'), self.suffix))
        try:
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                s.settimeout(self.timeout)
                s.bind((self.HOST, self.PORT))
                s.listen()
                conn, addr = s.accept()
                with conn:
                    conn.sendall(message)
                    print('Sent', repr(message))
                    if self.echo == True:
                        data = conn.recv(1024)
                        print('Received back', repr(data))
                    else: data = '0'.encode('utf-8')
        
        except socket.timeout as ex:
            print(ex)
            data = self.timeout_message.encode('utf-8')
        except ConnectionRefusedError as ex:
            print(ex)
            data = self.connection_refused_message.encode('utf-8')
        except ConnectionResetError as ex:
            print(ex)
            data = self.connection_reset_message.encode('utf-8')

        return data.replace(self.prefix,b'').replace(self.suffix,b'').decode('utf-8')
                
    def server_receive(self, response = None):

        try:
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                s.settimeout(self.timeout)
                s.bind((self.HOST, self.PORT))
                s.listen()
                conn, addr = s.accept()
                with conn:
                    print('Connected by', addr)
                    while True:
                        _data = conn.recv(1024)
                        if _data:
                            print('Received', repr(_data))
                            data = _data
                        elif not _data:
                            break
                        
                        if self.echo == True:
                            conn.sendall(data)
                            print('Sent echo', repr(data))
                        
                        if response is not None:
                            conn.sendall(response.encode('utf-8'))
                            print('Sent response:', repr(response))
        
        except socket.timeout as ex:
            print(ex)
            data = self.timeout_message.encode('utf-8')
        except ConnectionRefusedError as ex:
            print(ex)
            data = self.connection_refused_message.encode('utf-8')
        except ConnectionResetError as ex:
            print(ex)
            data = self.connection_reset_message.encode('utf-8')
            
        return data.replace(self.prefix, b'').replace(self.suffix, b'').decode('utf-8')

    def client_send(self, message):
        message = b''.join((self.prefix, message.encode('utf-8'), self.suffix))
        
        try:
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                s.settimeout(self.timeout)
                s.connect((self.HOST, self.PORT))
                s.sendall(message)
                print('Sent', repr(message))
                
                if self.echo == True:
                    data = s.recv(1024)
                    print('Received back', repr(data))
                else: data = '0'.encode('utf-8')
        
        except socket.timeout as ex:
            print(ex)
            data = self.timeout_message.encode('utf-8')
        except ConnectionRefusedError as ex:
            print(ex)
            data = self.connection_refused_message.encode('utf-8')
        except ConnectionResetError as ex:
            print(ex)
            data = self.connection_reset_message.encode('utf-8')
            
        return data.replace(self.prefix, b'').replace(self.suffix, b'').decode('utf-8')

    def client_receive(self, response = None):

        try:
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                s.settimeout(self.timeout)
                s.connect((self.HOST, self.PORT))
                while True:
                    _data = s.recv(1024)
                    if _data:
                        print('Received', repr(_data))
                        data = _data
                    elif not _data:
                        break
                    
                    if self.echo == True:
                        s.sendall(data)
                        print('Sent echo', repr(data))
                    
                    if response is not None:
                        s.sendall(response.encode('utf-8'))
                        print('Sent response:', repr(response))
        
        except socket.timeout as ex:
            print(ex)
            data = self.timeout_message.encode('utf-8')
        except ConnectionRefusedError as ex:
            print(ex)
            data = self.connection_refused_message.encode('utf-8')
        except ConnectionResetError as ex:
            print(ex)
            data = self.connection_reset_message.encode('utf-8')
            
        return data.replace(self.prefix, b'').replace(self.suffix, b'').decode('utf-8')
        
class Widget:
    
    def __init__(self, master):

        self.si = SocketInterface(echo = True)
        self.master = master
        self.master.call('wm', 'attributes', '.', '-topmost', True)
         
        # Title
        self.master.title('Test Socket Communication')
  
        # Create a main frame
        self.main_frame = Frame(self.master)
        self.main_frame.config(borderwidth = 2, relief = GROOVE)
        self.main_frame.grid(row = 0, column = 0, columnspan = 2)
  
        row = 0
        # Create a widgets within the frame
        self.config_label = ttk.Label(self.main_frame, text = 'Config')
        self.config_label.grid(row = row, column = 0, columnspan = 2)        
        
        row += 1
        # PLC_IP options
        self.PLC_IP_label = ttk.Label(self.main_frame, text = 'Server IP: ')
        self.PLC_IP_label.grid(row = row, column = 0, columnspan = 1)
        self.PLC_IP_str = StringVar()
        self.PLC_IP_str.set(self.si.HOST)
        self.PLC_IP_field = Entry(self.main_frame, textvariable = self.PLC_IP_str)
        self.PLC_IP_field.grid(row = row, column = 1)
       
        row += 1
       # PLC_PORT options
        self.PLC_PORT_label = ttk.Label(self.main_frame, text = 'Server Port: ')
        self.PLC_PORT_label.grid(row = row, column = 0, columnspan = 1)
        self.PLC_PORT_int = IntVar()
        self.PLC_PORT_int.set(self.si.PORT)
        self.PLC_PORT_field = Entry(self.main_frame, textvariable = self.PLC_PORT_int)
        self.PLC_PORT_field.grid(row = row, column = 1)

        row += 1
       # OUT_MESSAGE options
        self.OUT_MESSAGE_label = ttk.Label(self.main_frame, text = 'OUT_MESSAGE: ')
        self.OUT_MESSAGE_label.grid(row = row, column = 0, columnspan = 1)
        self.OUT_MESSAGE_str = StringVar()
        self.OUT_MESSAGE_str.set('Out Message')
        self.OUT_MESSAGE_field = Entry(self.main_frame, textvariable = self.OUT_MESSAGE_str)
        self.OUT_MESSAGE_field.grid(row = row, column = 1)
        
        row += 1
       # IN_MESSAGE options
        self.IN_MESSAGE_label = ttk.Label(self.main_frame, text = 'IN_MESSAGE: ')
        self.IN_MESSAGE_label.grid(row = row, column = 0, columnspan = 1)
        self.IN_MESSAGE_str = StringVar()
        self.IN_MESSAGE_str.set('In Message')
        self.IN_MESSAGE_field = Entry(self.main_frame, textvariable = self.IN_MESSAGE_str, state=DISABLED)
        self.IN_MESSAGE_field.grid(row = row, column = 1)        
        
        row += 1
       # TIMEOUT options
        self.TIMEOUT_label = ttk.Label(self.main_frame, text = 'TIMEOUT (s): ')
        self.TIMEOUT_label.grid(row = row, column = 0, columnspan = 1)
        self.TIMEOUT_int = IntVar()
        self.TIMEOUT_int.set(self.si.timeout)
        self.TIMEOUT_field = Entry(self.main_frame, textvariable = self.TIMEOUT_int)
        self.TIMEOUT_field.grid(row = row, column = 1)
        
        row += 1
        self.CLIENT_SEND = ttk.Button(self.main_frame, text = 'Client Send', command = self.client_send_wrapper, state = NORMAL)
        self.CLIENT_SEND.grid(row = row, column = 0)        
        
        row += 0
        self.CLIENT_SEND = ttk.Button(self.main_frame, text = 'Client Receive', command = self.client_receive_wrapper, state = NORMAL)
        self.CLIENT_SEND.grid(row = row, column = 1)
        
        row += 1
        self.CLIENT_SEND = ttk.Button(self.main_frame, text = 'Server Send', command = self.server_send_wrapper, state = NORMAL)
        self.CLIENT_SEND.grid(row = row, column = 0)
        
        row += 0
        self.CLIENT_SEND = ttk.Button(self.main_frame, text = 'Server Receive', command = self.server_receive_wrapper, state = NORMAL)
        self.CLIENT_SEND.grid(row = row, column = 1)
        
    def client_send_wrapper(self):
        print('Client sending...')
        self.si.HOST = str(self.PLC_IP_str.get())
        self.si.PORT = int(self.PLC_PORT_int.get())
        self.si.timeout = int(self.TIMEOUT_int.get())
        
        message = str(self.OUT_MESSAGE_str.get())
        
        echo = self.si.client_send(message)
        self.IN_MESSAGE_str.set(echo)
               
    def client_receive_wrapper(self):
        print('Client receiving...')
        self.si.HOST = str(self.PLC_IP_str.get())
        self.si.PORT = int(self.PLC_PORT_int.get())
        self.si.timeout = int(self.TIMEOUT_int.get())
        
        message = self.si.client_receive()
        self.IN_MESSAGE_str.set(message)
        
    def server_send_wrapper(self):
        print('Server sending...')
        self.si.HOST = str(self.PLC_IP_str.get())
        self.si.PORT = int(self.PLC_PORT_int.get())
        self.si.timeout = int(self.TIMEOUT_int.get())
        
        message = str(self.OUT_MESSAGE_str.get())
        
        echo = self.si.server_send(message)
        self.IN_MESSAGE_str.set(echo)
     
    def server_receive_wrapper(self):
        print('Server receiving...')
        self.si.HOST = str(self.PLC_IP_str.get())
        self.si.PORT = int(self.PLC_PORT_int.get())
        self.si.timeout = int(self.TIMEOUT_int.get())
        
        message = self.si.server_receive()
        
        self.IN_MESSAGE_str.set(message)

if __name__ == "__main__":
    root = Tk()
    app = Widget(root)
    root.mainloop()